@@ -1,13 +1,3 @@
-# -*- coding: utf-8 -*-
-"""
-    werkzeug.debug
-    ~~~~~~~~~~~~~~
-
-    WSGI application traceback debugger.
-
-    :copyright: 2007 Pallets
-    :license: BSD-3-Clause
-"""
 import getpass
 import hashlib
 import json
@@ -17,122 +7,131 @@ import pkgutil
 import re
 import sys
 import time
+import typing as t
 import uuid
 from itertools import chain
 from os.path import basename
 from os.path import join
 
-from .._compat import text_type
 from .._internal import _log
 from ..http import parse_cookie
 from ..security import gen_salt
-from ..wrappers import BaseRequest as Request
-from ..wrappers import BaseResponse as Response
+from ..wrappers.request import Request
+from ..wrappers.response import Response
 from .console import Console
-from .repr import debug_repr as _debug_repr
+from .tbtools import Frame
 from .tbtools import get_current_traceback
 from .tbtools import render_console_html
+from .tbtools import Traceback
 
-
-def debug_repr(*args, **kwargs):
-    import warnings
-
-    warnings.warn(
-        "'debug_repr' has moved to 'werkzeug.debug.repr.debug_repr'"
-        " as of version 0.7. This old import will be removed in version"
-        " 1.0.",
-        DeprecationWarning,
-        stacklevel=2,
-    )
-    return _debug_repr(*args, **kwargs)
-
+if t.TYPE_CHECKING:
+    from wsgiref.types import StartResponse
+    from wsgiref.types import WSGIApplication
+    from wsgiref.types import WSGIEnvironment
 
 # A week
 PIN_TIME = 60 * 60 * 24 * 7
 
 
-def hash_pin(pin):
-    if isinstance(pin, text_type):
-        pin = pin.encode("utf-8", "replace")
-    return hashlib.md5(pin + b"shittysalt").hexdigest()[:12]
+def hash_pin(pin: str) -> str:
+    return hashlib.sha1(f"{pin} added salt".encode("utf-8", "replace")).hexdigest()[:12]
 
 
-_machine_id = None
+_machine_id: t.Optional[t.Union[str, bytes]] = None
 
 
-def get_machine_id():
+def get_machine_id() -> t.Optional[t.Union[str, bytes]]:
     global _machine_id
-    rv = _machine_id
-    if rv is not None:
-        return rv
 
-    def _generate():
-        # Potential sources of secret information on linux.  The machine-id
-        # is stable across boots, the boot id is not
+    if _machine_id is not None:
+        return _machine_id
+
+    def _generate() -> t.Optional[t.Union[str, bytes]]:
+        linux = b""
+
+        # machine-id is stable across boots, boot_id is not.
         for filename in "/etc/machine-id", "/proc/sys/kernel/random/boot_id":
             try:
                 with open(filename, "rb") as f:
-                    return f.readline().strip()
-            except IOError:
+                    value = f.readline().strip()
+            except OSError:
                 continue
 
-        # On OS X we can use the computer's serial number assuming that
-        # ioreg exists and can spit out that information.
+            if value:
+                linux += value
+                break
+
+        # Containers share the same machine id, add some cgroup
+        # information. This is used outside containers too but should be
+        # relatively stable across boots.
         try:
-            # Also catch import errors: subprocess may not be available, e.g.
-            # Google App Engine
-            # See https://github.com/pallets/werkzeug/issues/925
+            with open("/proc/self/cgroup", "rb") as f:
+                linux += f.readline().strip().rpartition(b"/")[2]
+        except OSError:
+            pass
+
+        if linux:
+            return linux
+
+        # On OS X, use ioreg to get the computer's serial number.
+        try:
+            # subprocess may not be available, e.g. Google App Engine
+            # https://github.com/pallets/werkzeug/issues/925
             from subprocess import Popen, PIPE
 
             dump = Popen(
                 ["ioreg", "-c", "IOPlatformExpertDevice", "-d", "2"], stdout=PIPE
             ).communicate()[0]
             match = re.search(b'"serial-number" = <([^>]+)', dump)
+
             if match is not None:
                 return match.group(1)
         except (OSError, ImportError):
             pass
 
-        # On Windows we can use winreg to get the machine guid
-        wr = None
+        # On Windows, use winreg to get the machine guid.
         try:
-            import winreg as wr
+            import winreg
         except ImportError:
+            pass
+        else:
             try:
-                import _winreg as wr
-            except ImportError:
-                pass
-        if wr is not None:
-            try:
-                with wr.OpenKey(
-                    wr.HKEY_LOCAL_MACHINE,
+                with winreg.OpenKey(
+                    winreg.HKEY_LOCAL_MACHINE,
                     "SOFTWARE\\Microsoft\\Cryptography",
                     0,
-                    wr.KEY_READ | wr.KEY_WOW64_64KEY,
+                    winreg.KEY_READ | winreg.KEY_WOW64_64KEY,
                 ) as rk:
-                    machineGuid, wrType = wr.QueryValueEx(rk, "MachineGuid")
-                    if wrType == wr.REG_SZ:
-                        return machineGuid.encode("utf-8")
-                    else:
-                        return machineGuid
-            except WindowsError:
+                    guid: t.Union[str, bytes]
+                    guid_type: int
+                    guid, guid_type = winreg.QueryValueEx(rk, "MachineGuid")
+
+                    if guid_type == winreg.REG_SZ:
+                        return guid.encode("utf-8")  # type: ignore
+
+                    return guid
+            except OSError:
                 pass
 
-    _machine_id = rv = _generate()
-    return rv
+        return None
+
+    _machine_id = _generate()
+    return _machine_id
 
 
-class _ConsoleFrame(object):
+class _ConsoleFrame:
     """Helper class so that we can reuse the frame console code for the
     standalone console.
     """
 
-    def __init__(self, namespace):
+    def __init__(self, namespace: t.Dict[str, t.Any]):
         self.console = Console(namespace)
         self.id = 0
 
 
-def get_pin_and_cookie_name(app):
+def get_pin_and_cookie_name(
+    app: "WSGIApplication",
+) -> t.Union[t.Tuple[str, str], t.Tuple[None, None]]:
     """Given an application object this returns a semi-stable 9 digit pin
     code and a random key.  The hope is that this is stable between
     restarts to not make debugging particularly frustrating.  If the pin
@@ -156,7 +155,8 @@ def get_pin_and_cookie_name(app):
         else:
             num = pin
 
-    modname = getattr(app, "__module__", getattr(app.__class__, "__module__"))
+    modname = getattr(app, "__module__", t.cast(object, app).__class__.__module__)
+    username: t.Optional[str]
 
     try:
         # getuser imports the pwd module, which does not exist in Google
@@ -173,7 +173,7 @@ def get_pin_and_cookie_name(app):
     probably_public_bits = [
         username,
         modname,
-        getattr(app, "__name__", getattr(app.__class__, "__name__")),
+        getattr(app, "__name__", type(app).__name__),
         getattr(mod, "__file__", None),
     ]
 
@@ -182,22 +182,22 @@ def get_pin_and_cookie_name(app):
     # within the unauthenticated debug page.
     private_bits = [str(uuid.getnode()), get_machine_id()]
 
-    h = hashlib.md5()
+    h = hashlib.sha1()
     for bit in chain(probably_public_bits, private_bits):
         if not bit:
             continue
-        if isinstance(bit, text_type):
+        if isinstance(bit, str):
             bit = bit.encode("utf-8")
         h.update(bit)
     h.update(b"cookiesalt")
 
-    cookie_name = "__wzd" + h.hexdigest()[:20]
+    cookie_name = f"__wzd{h.hexdigest()[:20]}"
 
     # If we need to generate a pin we salt it a bit more so that we don't
     # end up with the same value and generate out 9 digits
     if num is None:
         h.update(b"pinsalt")
-        num = ("%09d" % int(h.hexdigest(), 16))[:9]
+        num = f"{int(h.hexdigest(), 16):09d}"[:9]
 
     # Format the pincode in groups of digits for easier remembering if
     # we don't have a result yet.
@@ -215,7 +215,7 @@ def get_pin_and_cookie_name(app):
     return rv, cookie_name
 
 
-class DebuggedApplication(object):
+class DebuggedApplication:
     """Enables debugging support for a given application::
 
         from werkzeug.debug import DebuggedApplication
@@ -225,9 +225,6 @@ class DebuggedApplication(object):
     The `evalex` keyword argument allows evaluating expressions in a
     traceback's frame context.
 
-    .. versionadded:: 0.9
-       The `lodgeit_url` parameter was deprecated.
-
     :param app: the WSGI application to run debugged.
     :param evalex: enable exception evaluation feature (interactive
                    debugging).  This requires a non-forking server.
@@ -245,34 +242,26 @@ class DebuggedApplication(object):
     :param pin_logging: enables the logging of the pin system.
     """
 
+    _pin: str
+    _pin_cookie: str
+
     def __init__(
         self,
-        app,
-        evalex=False,
-        request_key="werkzeug.request",
-        console_path="/console",
-        console_init_func=None,
-        show_hidden_frames=False,
-        lodgeit_url=None,
-        pin_security=True,
-        pin_logging=True,
-    ):
-        if lodgeit_url is not None:
-            from warnings import warn
-
-            warn(
-                "'lodgeit_url' is no longer used as of version 0.9 and"
-                " will be removed in version 1.0. Werkzeug uses"
-                " https://gist.github.com/ instead.",
-                DeprecationWarning,
-                stacklevel=2,
-            )
+        app: "WSGIApplication",
+        evalex: bool = False,
+        request_key: str = "werkzeug.request",
+        console_path: str = "/console",
+        console_init_func: t.Optional[t.Callable[[], t.Dict[str, t.Any]]] = None,
+        show_hidden_frames: bool = False,
+        pin_security: bool = True,
+        pin_logging: bool = True,
+    ) -> None:
         if not console_init_func:
             console_init_func = None
         self.app = app
         self.evalex = evalex
-        self.frames = {}
-        self.tracebacks = {}
+        self.frames: t.Dict[int, t.Union[Frame, _ConsoleFrame]] = {}
+        self.tracebacks: t.Dict[int, Traceback] = {}
         self.request_key = request_key
         self.console_path = console_path
         self.console_init_func = console_init_func
@@ -288,40 +277,42 @@ class DebuggedApplication(object):
                 if self.pin is None:
                     _log("warning", " * Debugger PIN disabled. DEBUGGER UNSECURED!")
                 else:
-                    _log("info", " * Debugger PIN: %s" % self.pin)
+                    _log("info", " * Debugger PIN: %s", self.pin)
         else:
             self.pin = None
 
-    def _get_pin(self):
+    @property
+    def pin(self) -> t.Optional[str]:
         if not hasattr(self, "_pin"):
-            self._pin, self._pin_cookie = get_pin_and_cookie_name(self.app)
+            pin_cookie = get_pin_and_cookie_name(self.app)
+            self._pin, self._pin_cookie = pin_cookie  # type: ignore
         return self._pin
 
-    def _set_pin(self, value):
+    @pin.setter
+    def pin(self, value: str) -> None:
         self._pin = value
 
-    pin = property(_get_pin, _set_pin)
-    del _get_pin, _set_pin
-
     @property
-    def pin_cookie_name(self):
+    def pin_cookie_name(self) -> str:
         """The name of the pin cookie."""
         if not hasattr(self, "_pin_cookie"):
-            self._pin, self._pin_cookie = get_pin_and_cookie_name(self.app)
+            pin_cookie = get_pin_and_cookie_name(self.app)
+            self._pin, self._pin_cookie = pin_cookie  # type: ignore
         return self._pin_cookie
 
-    def debug_application(self, environ, start_response):
+    def debug_application(
+        self, environ: "WSGIEnvironment", start_response: "StartResponse"
+    ) -> t.Iterator[bytes]:
         """Run the application and conserve the traceback frames."""
         app_iter = None
         try:
             app_iter = self.app(environ, start_response)
-            for item in app_iter:
-                yield item
+            yield from app_iter
             if hasattr(app_iter, "close"):
-                app_iter.close()
+                app_iter.close()  # type: ignore
         except Exception:
             if hasattr(app_iter, "close"):
-                app_iter.close()
+                app_iter.close()  # type: ignore
             traceback = get_current_traceback(
                 skip=1,
                 show_hidden_frames=self.show_hidden_frames,
@@ -359,11 +350,13 @@ class DebuggedApplication(object):
 
             traceback.log(environ["wsgi.errors"])
 
-    def execute_command(self, request, command, frame):
+    def execute_command(
+        self, request: Request, command: str, frame: t.Union[Frame, _ConsoleFrame]
+    ) -> Response:
         """Execute a command in a console."""
         return Response(frame.console.eval(command), mimetype="text/html")
 
-    def display_console(self, request):
+    def display_console(self, request: Request) -> Response:
         """Display a standalone shell."""
         if 0 not in self.frames:
             if self.console_init_func is None:
@@ -378,12 +371,7 @@ class DebuggedApplication(object):
             mimetype="text/html",
         )
 
-    def paste_traceback(self, request, traceback):
-        """Paste the traceback and return a JSON response."""
-        rv = traceback.paste()
-        return Response(json.dumps(rv), mimetype="application/json")
-
-    def get_resource(self, request, filename):
+    def get_resource(self, request: Request, filename: str) -> Response:
         """Return a static resource from the shared folder."""
         filename = join("shared", basename(filename))
         try:
@@ -395,7 +383,7 @@ class DebuggedApplication(object):
             return Response(data, mimetype=mimetype)
         return Response("Not Found", status=404)
 
-    def check_pin_trust(self, environ):
+    def check_pin_trust(self, environ: "WSGIEnvironment") -> t.Optional[bool]:
         """Checks if the request passed the pin test.  This returns `True` if the
         request is trusted on a pin/cookie basis and returns `False` if not.
         Additionally if the cookie's stored pin hash is wrong it will return
@@ -413,15 +401,16 @@ class DebuggedApplication(object):
             return None
         return (time.time() - PIN_TIME) < int(ts)
 
-    def _fail_pin_auth(self):
+    def _fail_pin_auth(self) -> None:
         time.sleep(5.0 if self._failed_pin_auth > 5 else 0.5)
         self._failed_pin_auth += 1
 
-    def pin_auth(self, request):
+    def pin_auth(self, request: Request) -> Response:
         """Authenticates with the pin."""
         exhausted = False
         auth = False
         trust = self.check_pin_trust(request.environ)
+        pin = t.cast(str, self.pin)
 
         # If the trust return value is `None` it means that the cookie is
         # set but the stored pin hash value is bad.  This means that the
@@ -443,8 +432,9 @@ class DebuggedApplication(object):
 
         # Otherwise go through pin based authentication
         else:
-            entered_pin = request.args.get("pin")
-            if entered_pin.strip().replace("-", "") == self.pin.replace("-", ""):
+            entered_pin = request.args["pin"]
+
+            if entered_pin.strip().replace("-", "") == pin.replace("-", ""):
                 self._failed_pin_auth = 0
                 auth = True
             else:
@@ -457,23 +447,26 @@ class DebuggedApplication(object):
         if auth:
             rv.set_cookie(
                 self.pin_cookie_name,
-                "%s|%s" % (int(time.time()), hash_pin(self.pin)),
+                f"{int(time.time())}|{hash_pin(pin)}",
                 httponly=True,
+                samesite="None",
             )
         elif bad_cookie:
             rv.delete_cookie(self.pin_cookie_name)
         return rv
 
-    def log_pin_request(self):
+    def log_pin_request(self) -> Response:
         """Log the pin if needed."""
         if self.pin_logging and self.pin is not None:
             _log(
                 "info", " * To enable the debugger you need to enter the security pin:"
             )
-            _log("info", " * Debugger pin code: %s" % self.pin)
+            _log("info", " * Debugger pin code: %s", self.pin)
         return Response("")
 
-    def __call__(self, environ, start_response):
+    def __call__(
+        self, environ: "WSGIEnvironment", start_response: "StartResponse"
+    ) -> t.Iterable[bytes]:
         """Dispatch the requests."""
         # important: don't ever access a function here that reads the incoming
         # form data!  Otherwise the application won't have access to that data
@@ -484,16 +477,13 @@ class DebuggedApplication(object):
             cmd = request.args.get("cmd")
             arg = request.args.get("f")
             secret = request.args.get("s")
-            traceback = self.tracebacks.get(request.args.get("tb", type=int))
             frame = self.frames.get(request.args.get("frm", type=int))
             if cmd == "resource" and arg:
-                response = self.get_resource(request, arg)
-            elif cmd == "paste" and traceback is not None and secret == self.secret:
-                response = self.paste_traceback(request, traceback)
+                response = self.get_resource(request, arg)  # type: ignore
             elif cmd == "pinauth" and secret == self.secret:
-                response = self.pin_auth(request)
+                response = self.pin_auth(request)  # type: ignore
             elif cmd == "printpin" and secret == self.secret:
-                response = self.log_pin_request()
+                response = self.log_pin_request()  # type: ignore
             elif (
                 self.evalex
                 and cmd is not None
@@ -501,11 +491,11 @@ class DebuggedApplication(object):
                 and self.secret == secret
                 and self.check_pin_trust(environ)
             ):
-                response = self.execute_command(request, cmd, frame)
+                response = self.execute_command(request, cmd, frame)  # type: ignore
         elif (
             self.evalex
             and self.console_path is not None
             and request.path == self.console_path
         ):
-            response = self.display_console(request)
+            response = self.display_console(request)  # type: ignore
         return response(environ, start_response)
